package ch.bailu.aat.services.icons;

import java.io.File;
import java.io.IOException;

import ch.bailu.aat.gpx.GpxAttributes;
import ch.bailu.aat.helpers.AppDirectory;
import ch.bailu.aat.helpers.AppLog;
import ch.bailu.aat.helpers.file.FileAccess;
import ch.bailu.aat.services.ServiceContext;
import ch.bailu.aat.services.VirtualService;

public class IconMapService extends VirtualService {
    private final String NKEY_KEY = "class";
    private final String NKEY_VALUE = "type";

    private final static String MAP_FILE="iconmap.txt";


    private final IconMap map;




    public IconMapService(ServiceContext sc) {
        super(sc);

        File directory = AppDirectory.getDataDirectory(getContext(), AppDirectory.DIR_OSM_FEATURES_ICONS);
        map = new IconMap(directory.toString());

        final File mapFile = new File(directory, MAP_FILE);

        if (mapFile.exists()) {
            try {
                new IconMapParser(new FileAccess(mapFile), map);
            } catch (IOException e) {
                AppLog.e(getContext(), this, e);
            }
        }
    }


    public String getIconPath(GpxAttributes attr) {

        for (int i=0; i<attr.size(); i++) {
            final String icon = getBigIconPath(attr.getKey(i), attr.getValue(i));
            if (icon != null) return icon;
        }
        return getIconPathFromNominatimTypeAttributes(attr);
    }


    public String getIconPathFromNominatimTypeAttributes(GpxAttributes attr) {
        String key = attr.get(NKEY_KEY);

        if (key != null) {
            String value = attr.get(NKEY_VALUE);
            if (value != null)
                return getBigIconPath(key, value);
        }
        return null;
    }



    private String getSmallIconPath(String key, String value) {
        final IconMap.Icon icon = map.get(key,value);

        if (icon == null) {
            return null;
        }

        return icon.small;
    }


    private String getBigIconPath(String key, String value) {
        final IconMap.Icon icon = map.get(key,value);

        if (icon == null) {
            return null;
        }

        return icon.big;
    }



    public void iconify(StringBuilder html, String key, String value) {
        String icon = getBigIconPath(key, value);

        if (icon != null) {
            html.append("<p><img src=\"");
            html.append(icon);
            html.append("\"/></p>");
        }
    }


    /*
    public void iconify(GpxList list) {
        new GpxIconifier().walkTrack(list);
    }
    */

    @Override
    public void close() {

    }

/*
    private class GpxIconifier extends GpxListWalker {

        @Override
        public boolean doList(GpxList track) {
            return track.getDelta().getType()==GpxType.WAY;
        }

        @Override
        public boolean doSegment(GpxSegmentNode segment) {
            return true;
        }

        @Override
        public boolean doMarker(GpxSegmentNode marker) {
            return true;
        }

        @Override
        public void doPoint(GpxPointNode point) {
            GpxAttributes a = point.getAttributes();

            for(int i = 0; a != null && i< a.size(); i++) {
                final String key = a.getKey(i);
                final String value = a.getTime(i);

                final String sicon = getSmallIconPath(key,value);
                final String bicon = getBigIconPath(key,value);

                if (sicon != null && bicon != null) {
                    a.put(KEY_ICON_SMALL, sicon);
                    a.put(KEY_ICON_BIG, bicon);
                    break;
                }
            }
        }
    }

*/

    @Override
    public void appendStatusText(StringBuilder builder) {
        // TODO Auto-generated method stub

    }
}
